@@ -14,14 +14,19 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *)
 
+module Ocaml_ident = Ident
+module Ocaml_env = Env
+
 open Names
 
 module Identifier = struct
   type 'a id = 'a Paths_types.id = { iv : 'a; ihash : int; ikey : string }
 
-  type t = Paths_types.Identifier.any
+  module Id = Paths_types.Identifier
 
-  type t_pv = Paths_types.Identifier.any_pv
+  type t = Id.any
+
+  type t_pv = Id.any_pv
 
   let rec name_aux : t -> string =
    fun x ->
@@ -38,6 +43,7 @@ module Identifier = struct
     | `Constructor (_, name) -> ConstructorName.to_string name
     | `Field (_, name) -> FieldName.to_string name
     | `Extension (_, name) -> ExtensionName.to_string name
+    | `ExtensionDecl (_, _, name) -> ExtensionName.to_string name
     | `Exception (_, name) -> ExceptionName.to_string name
     | `CoreException name -> ExceptionName.to_string name
     | `Value (_, name) -> ValueName.to_string name
@@ -46,14 +52,101 @@ module Identifier = struct
     | `Method (_, name) -> MethodName.to_string name
     | `InstanceVariable (_, name) -> InstanceVariableName.to_string name
     | `Label (_, name) -> LabelName.to_string name
+    | `SourcePage (dir, name) -> name_aux (dir :> t) ^ name
+    | `SourceDir (({ iv = `SourceDir _; _ } as p), n) ->
+        name_aux (p :> t) ^ n ^ "/"
+    | `SourceDir (_, n) -> "./" ^ n ^ "/"
+    | `SourceLocation (x, anchor) ->
+        name_aux (x :> t) ^ "#" ^ DefName.to_string anchor
+    | `SourceLocationMod x -> name_aux (x :> t)
+    | `SourceLocationInternal (x, anchor) ->
+        name_aux (x :> t) ^ "#" ^ LocalName.to_string anchor
+    | `AssetFile (_, name) -> name
+
+  let rec is_internal : t -> bool =
+   fun x ->
+    match x.iv with
+    | `Root (_, name) -> ModuleName.is_internal name
+    | `Page (_, _) -> false
+    | `LeafPage (_, _) -> false
+    | `Module (_, name) -> ModuleName.is_internal name
+    | `Parameter (_, name) -> ModuleName.is_internal name
+    | `Result x -> is_internal (x :> t)
+    | `ModuleType (_, name) -> ModuleTypeName.is_internal name
+    | `Type (_, name) -> TypeName.is_internal name
+    | `CoreType name -> TypeName.is_internal name
+    | `Constructor (parent, _) -> is_internal (parent :> t)
+    | `Field (parent, _) -> is_internal (parent :> t)
+    | `Extension (parent, _) -> is_internal (parent :> t)
+    | `ExtensionDecl (parent, _, _) -> is_internal (parent :> t)
+    | `Exception (parent, _) -> is_internal (parent :> t)
+    | `CoreException _ -> false
+    | `Value (_, name) -> ValueName.is_internal name
+    | `Class (_, name) -> ClassName.is_internal name
+    | `ClassType (_, name) -> ClassTypeName.is_internal name
+    | `Method (parent, _) -> is_internal (parent :> t)
+    | `InstanceVariable (parent, _) -> is_internal (parent :> t)
+    | `Label (parent, _) -> is_internal (parent :> t)
+    | `SourceDir _ | `SourceLocationMod _ | `SourceLocation _ | `SourcePage _
+    | `SourceLocationInternal _ | `AssetFile _ ->
+        false
 
   let name : [< t_pv ] id -> string = fun n -> name_aux (n :> t)
 
+  let rec full_name_aux : t -> string list =
+   fun x ->
+    match x.iv with
+    | `Root (_, name) -> [ ModuleName.to_string name ]
+    | `Page (_, name) -> [ PageName.to_string name ]
+    | `LeafPage (_, name) -> [ PageName.to_string name ]
+    | `Module (parent, name) ->
+        ModuleName.to_string name :: full_name_aux (parent :> t)
+    | `Parameter (parent, name) ->
+        ModuleName.to_string name :: full_name_aux (parent :> t)
+    | `Result x -> full_name_aux (x :> t)
+    | `ModuleType (parent, name) ->
+        ModuleTypeName.to_string name :: full_name_aux (parent :> t)
+    | `Type (parent, name) ->
+        TypeName.to_string name :: full_name_aux (parent :> t)
+    | `CoreType name -> [ TypeName.to_string name ]
+    | `Constructor (parent, name) ->
+        ConstructorName.to_string name :: full_name_aux (parent :> t)
+    | `Field (parent, name) ->
+        FieldName.to_string name :: full_name_aux (parent :> t)
+    | `Extension (parent, name) ->
+        ExtensionName.to_string name :: full_name_aux (parent :> t)
+    | `ExtensionDecl (parent, _, name) ->
+        ExtensionName.to_string name :: full_name_aux (parent :> t)
+    | `Exception (parent, name) ->
+        ExceptionName.to_string name :: full_name_aux (parent :> t)
+    | `CoreException name -> [ ExceptionName.to_string name ]
+    | `Value (parent, name) ->
+        ValueName.to_string name :: full_name_aux (parent :> t)
+    | `Class (parent, name) ->
+        ClassName.to_string name :: full_name_aux (parent :> t)
+    | `ClassType (parent, name) ->
+        ClassTypeName.to_string name :: full_name_aux (parent :> t)
+    | `Method (parent, name) ->
+        MethodName.to_string name :: full_name_aux (parent :> t)
+    | `InstanceVariable (parent, name) ->
+        InstanceVariableName.to_string name :: full_name_aux (parent :> t)
+    | `Label (parent, name) ->
+        LabelName.to_string name :: full_name_aux (parent :> t)
+    | `AssetFile (parent, name) -> name :: full_name_aux (parent :> t)
+    | `SourceDir _ | `SourceLocationMod _ | `SourceLocation _ | `SourcePage _
+    | `SourceLocationInternal _ ->
+        []
+
+  let fullname : [< t_pv ] id -> string list =
+   fun n -> List.rev @@ full_name_aux (n :> t)
+
+  let is_internal : [< t_pv ] id -> bool = fun n -> is_internal (n :> t)
+
   let rec label_parent_aux =
-    let open Paths_types.Identifier in
-    fun (n : any) ->
+    let open Id in
+    fun (n : non_src) ->
       match n with
-      | { iv = `Result i; _ } -> label_parent_aux (i :> any)
+      | { iv = `Result i; _ } -> label_parent_aux (i :> non_src)
       | { iv = `CoreType _; _ } | { iv = `CoreException _; _ } -> assert false
       | { iv = `Root _; _ } as p -> (p :> label_parent)
       | { iv = `Page _; _ } as p -> (p :> label_parent)
@@ -65,6 +158,7 @@ module Identifier = struct
       | { iv = `ClassType (p, _); _ }
       | { iv = `Type (p, _); _ }
       | { iv = `Extension (p, _); _ }
+      | { iv = `ExtensionDecl (p, _, _); _ }
       | { iv = `Exception (p, _); _ }
       | { iv = `Value (p, _); _ } ->
           (p : signature :> label_parent)
@@ -72,9 +166,9 @@ module Identifier = struct
       | { iv = `Method (p, _); _ } | { iv = `InstanceVariable (p, _); _ } ->
           (p : class_signature :> label_parent)
       | { iv = `Constructor (p, _); _ } -> (p : datatype :> label_parent)
-      | { iv = `Field (p, _); _ } -> (p : parent :> label_parent)
+      | { iv = `Field (p, _); _ } -> (p : field_parent :> label_parent)
 
-  let label_parent n = label_parent_aux (n :> t)
+  let label_parent n = label_parent_aux (n :> Id.non_src)
 
   let equal x y = x.ihash = y.ihash && x.ikey = y.ikey
 
@@ -84,176 +178,129 @@ module Identifier = struct
 
   type any = t
 
+  type any_pv = t_pv
+
+  module type IdSig = sig
+    type t
+    type t_pv
+    val equal : t -> t -> bool
+    val hash : t -> int
+    val compare : t -> t -> int
+  end
+
   module Any = struct
     type t = any
-
+    type t_pv = any_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Signature = struct
-    type t = Paths_types.Identifier.signature
-
-    type t_pv = Paths_types.Identifier.signature_pv
-
+    type t = Id.signature
+    type t_pv = Id.signature_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module ClassSignature = struct
-    type t = Paths_types.Identifier.class_signature
-
-    type t_pv = Paths_types.Identifier.class_signature_pv
-
+    type t = Id.class_signature
+    type t_pv = Id.class_signature_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module DataType = struct
-    type t = Paths_types.Identifier.datatype
-
-    type t_pv = Paths_types.Identifier.datatype_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.datatype
+    type t_pv = Id.datatype_pv
   end
 
-  module Parent = struct
-    type t = Paths_types.Identifier.parent
-
-    type t_pv = Paths_types.Identifier.parent_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+  module FieldParent = struct
+    type t = Paths_types.Identifier.field_parent
+    type t_pv = Paths_types.Identifier.field_parent_pv
   end
 
   module LabelParent = struct
-    type t = Paths_types.Identifier.label_parent
-
-    type t_pv = Paths_types.Identifier.label_parent_pv
-
+    type t = Id.label_parent
+    type t_pv = Id.label_parent_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module RootModule = struct
-    type t = Paths_types.Identifier.root_module
-
-    type t_pv = Paths_types.Identifier.root_module_pv
-
+    type t = Id.root_module
+    type t_pv = Id.root_module_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Module = struct
-    type t = Paths_types.Identifier.module_
-
-    type t_pv = Paths_types.Identifier.module_pv
-
+    type t = Id.module_
+    type t_pv = Id.module_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module FunctorParameter = struct
-    type t = Paths_types.Identifier.functor_parameter
-
-    type t_pv = Paths_types.Identifier.functor_parameter_pv
-
+    type t = Id.functor_parameter
+    type t_pv = Id.functor_parameter_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
+
+    let functor_arg_pos { iv = `Parameter (p, _); _ } =
+      let rec inner_sig = function
+        | `Result { iv = p; _ } -> 1 + inner_sig p
+        | `Module _ | `ModuleType _ | `Root _ | `Parameter _ -> 1
+      in
+      inner_sig p.iv
   end
 
   module FunctorResult = struct
-    type t = Paths_types.Identifier.functor_result
-
-    type t_pv = Paths_types.Identifier.functor_result_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.functor_result
+    type t_pv = Id.functor_result_pv
   end
 
   module ModuleType = struct
-    type t = Paths_types.Identifier.module_type
-
-    type t_pv = Paths_types.Identifier.module_type_pv
-
+    type t = Id.module_type
+    type t_pv = Id.module_type_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Type = struct
-    type t = Paths_types.Identifier.type_
-
-    type t_pv = Paths_types.Identifier.type_pv
-
+    type t = Id.type_
+    type t_pv = Id.type_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Constructor = struct
-    type t = Paths_types.Identifier.constructor
-
-    type t_pv = Paths_types.Identifier.constructor_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.constructor
+    type t_pv = Id.constructor_pv
   end
 
   module Field = struct
-    type t = Paths_types.Identifier.field
-
-    type t_pv = Paths_types.Identifier.field_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.field
+    type t_pv = Id.field_pv
   end
 
   module Extension = struct
-    type t = Paths_types.Identifier.extension
+    type t = Id.extension
+    type t_pv = Id.extension_pv
+  end
 
-    type t_pv = Paths_types.Identifier.extension_pv
+  module ExtensionDecl = struct
+    type t = Paths_types.Identifier.extension_decl
+
+    type t_pv = Paths_types.Identifier.extension_decl_pv
 
     let equal = equal
 
@@ -263,173 +310,150 @@ module Identifier = struct
   end
 
   module Exception = struct
-    type t = Paths_types.Identifier.exception_
-
-    type t_pv = Paths_types.Identifier.exception_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.exception_
+    type t_pv = Id.exception_pv
   end
 
   module Value = struct
-    type t = Paths_types.Identifier.value
-
-    type t_pv = Paths_types.Identifier.value_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.value
+    type t_pv = Id.value_pv
   end
 
   module Class = struct
-    type t = Paths_types.Identifier.class_
-
-    type t_pv = Paths_types.Identifier.class_pv
-
+    type t = Id.class_
+    type t_pv = Id.class_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module ClassType = struct
-    type t = Paths_types.Identifier.class_type
-
-    type t_pv = Paths_types.Identifier.class_type_pv
-
+    type t = Id.class_type
+    type t_pv = Id.class_type_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Method = struct
-    type t = Paths_types.Identifier.method_
-
-    type t_pv = Paths_types.Identifier.method_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.method_
+    type t_pv = Id.method_pv
   end
 
   module InstanceVariable = struct
-    type t = Paths_types.Identifier.instance_variable
-
-    type t_pv = Paths_types.Identifier.instance_variable_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.instance_variable
+    type t_pv = Id.instance_variable_pv
   end
 
   module Label = struct
     type t = Paths_types.Identifier.label
-
     type t_pv = Paths_types.Identifier.label_pv
-
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
   module Page = struct
-    type t = Paths_types.Identifier.page
-
-    type t_pv = Paths_types.Identifier.page_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.page
+    type t_pv = Id.page_pv
   end
 
   module ContainerPage = struct
-    type t = Paths_types.Identifier.container_page
+    type t = Id.container_page
+    type t_pv = Id.container_page_pv
+  end
 
-    type t_pv = Paths_types.Identifier.container_page_pv
+  module NonSrc = struct
+    type t = Paths_types.Identifier.non_src
+    type t_pv = Paths_types.Identifier.non_src_pv
+  end
 
+  module SourceDir = struct
+    type t = Id.source_dir
+    type t_pv = Id.source_dir_pv
     let equal = equal
-
     let hash = hash
-
     let compare = compare
   end
 
+  module SourcePage = struct
+    type t = Id.source_page
+    type t_pv = Id.source_page_pv
+  end
+
+  module SourceLocation = struct
+    type t = Paths_types.Identifier.source_location
+    type t_pv = Paths_types.Identifier.source_location_pv
+  end
+
+  module AssetFile = struct
+    type t = Id.asset_file
+    type t_pv = Id.asset_file_pv
+  end
+
   module OdocId = struct
-    type t = Paths_types.Identifier.odoc_id
-
-    type t_pv = Paths_types.Identifier.odoc_id_pv
-
-    let equal = equal
-
-    let hash = hash
-
-    let compare = compare
+    type t = Id.odoc_id
+    type t_pv = Id.odoc_id_pv
   end
 
   module Path = struct
     module Module = struct
-      type t = Paths_types.Identifier.path_module
-
-      type t_pv = Paths_types.Identifier.path_module_pv
-
+      type t = Id.path_module
+      type t_pv = Id.path_module_pv
       let equal = equal
-
       let hash = hash
-
       let compare = compare
     end
 
     module ModuleType = struct
-      type t = Paths_types.Identifier.path_module_type
-
+      type t = Id.path_module_type
+      type t_pv = Id.module_type_pv
       let equal = equal
-
       let hash = hash
-
       let compare = compare
     end
 
     module Type = struct
-      type t = Paths_types.Identifier.path_type
-
-      type t_pv = Paths_types.Identifier.path_type_pv
-
+      type t = Id.path_type
+      type t_pv = Id.path_type_pv
       let equal = equal
-
       let hash = hash
+      let compare = compare
+    end
 
+    module DataType = struct
+      type t = Id.path_datatype
+      type t_pv = Id.path_datatype_pv
+      let equal = equal
+      let hash = hash
+      let compare = compare
+    end
+
+    module Constructor = struct
+      type t = Id.path_constructor
+      type t_pv = Id.constructor_pv
+      let equal = equal
+      let hash = hash
+      let compare = compare
+    end
+
+    module Value = struct
+      type t = Id.path_value
+      type t_pv = Id.value_pv
+      let equal = equal
+      let hash = hash
       let compare = compare
     end
 
     module ClassType = struct
-      type t = Paths_types.Identifier.path_class_type
-
-      type t_pv = Paths_types.Identifier.path_class_type_pv
-
+      type t = Id.path_class_type
+      type t_pv = Id.path_class_type_pv
       let equal = equal
-
       let hash = hash
-
       let compare = compare
     end
 
-    type t = Paths_types.Identifier.path_any
+    type t = Id.path_any
   end
 
   module Maps = struct
@@ -481,6 +505,31 @@ module Identifier = struct
         [> `LeafPage of ContainerPage.t option * PageName.t ] id =
       mk_parent_opt PageName.to_string "lp" (fun (p, n) -> `LeafPage (p, n))
 
+    let asset_file : Page.t * string -> AssetFile.t =
+      mk_parent (fun k -> k) "asset" (fun (p, n) -> `AssetFile (p, n))
+
+    let source_page (container_page, path) =
+      let rec source_dir dir =
+        match dir with
+        | [] -> (container_page : ContainerPage.t :> SourceDir.t)
+        | a :: q ->
+            let parent = source_dir q in
+            mk_parent
+              (fun k -> k)
+              "sd"
+              (fun (p, dir) -> `SourceDir (p, dir))
+              (parent, a)
+      in
+      match List.rev path with
+      | [] -> assert false
+      | file :: dir ->
+          let parent = source_dir dir in
+          mk_parent
+            (fun x -> x)
+            "sp"
+            (fun (p, rp) -> `SourcePage (p, rp))
+            (parent, file)
+
     let root :
         ContainerPage.t option * ModuleName.t ->
         [> `Root of ContainerPage.t option * ModuleName.t ] id =
@@ -523,13 +572,14 @@ module Identifier = struct
       mk_fresh (fun s -> s) "coret" (fun s -> `CoreType (TypeName.make_std s))
 
     let constructor :
-        Type.t * ConstructorName.t ->
-        [> `Constructor of Type.t * ConstructorName.t ] id =
+        DataType.t * ConstructorName.t ->
+        [> `Constructor of DataType.t * ConstructorName.t ] id =
       mk_parent ConstructorName.to_string "ctor" (fun (p, n) ->
           `Constructor (p, n))
 
     let field :
-        Parent.t * FieldName.t -> [> `Field of Parent.t * FieldName.t ] id =
+        FieldParent.t * FieldName.t ->
+        [> `Field of FieldParent.t * FieldName.t ] id =
       mk_parent FieldName.to_string "fld" (fun (p, n) -> `Field (p, n))
 
     let extension :
@@ -537,6 +587,16 @@ module Identifier = struct
         [> `Extension of Signature.t * ExtensionName.t ] id =
       mk_parent ExtensionName.to_string "extn" (fun (p, n) -> `Extension (p, n))
 
+    let extension_decl :
+        Signature.t * (ExtensionName.t * ExtensionName.t) ->
+        [> `ExtensionDecl of Signature.t * ExtensionName.t * ExtensionName.t ]
+        id =
+      mk_parent
+        (fun (n, m) ->
+          ExtensionName.to_string n ^ "." ^ ExtensionName.to_string m)
+        "extn-decl"
+        (fun (p, (n, m)) -> `ExtensionDecl (p, n, m))
+
     let exception_ :
         Signature.t * ExceptionName.t ->
         [> `Exception of Signature.t * ExceptionName.t ] id =
@@ -568,6 +628,26 @@ module Identifier = struct
         LabelParent.t * LabelName.t ->
         [> `Label of LabelParent.t * LabelName.t ] id =
       mk_parent LabelName.to_string "l" (fun (p, n) -> `Label (p, n))
+
+    let source_location :
+        SourcePage.t * DefName.t ->
+        [> `SourceLocation of SourcePage.t * DefName.t ] id =
+      mk_parent DefName.to_string "sl" (fun (p, n) -> `SourceLocation (p, n))
+
+    let source_location_mod :
+        SourcePage.t -> [> `SourceLocationMod of SourcePage.t ] id =
+     fun s ->
+      mk_parent
+        (fun () -> "__slm__")
+        ""
+        (fun (s, ()) -> `SourceLocationMod s)
+        (s, ())
+
+    let source_location_int :
+        SourcePage.t * LocalName.t ->
+        [> `SourceLocationInternal of SourcePage.t * LocalName.t ] id =
+      mk_parent LocalName.to_string "sli" (fun (p, n) ->
+          `SourceLocationInternal (p, n))
   end
 end
 
@@ -601,6 +681,9 @@ module Path = struct
       | `ModuleType (p, _) -> inner (p : module_ :> any)
       | `Type (_, t) when Names.TypeName.is_internal t -> true
       | `Type (p, _) -> inner (p : module_ :> any)
+      | `Value (_, t) when Names.ValueName.is_internal t -> true
+      | `Value (p, _) -> inner (p : module_ :> any)
+      | `Constructor (p, _) -> inner (p : datatype :> any)
       | `Class (p, _) -> inner (p : module_ :> any)
       | `ClassType (p, _) -> inner (p : module_ :> any)
       | `Alias (dest, `Resolved src) ->
@@ -615,6 +698,8 @@ module Path = struct
       | `CanonicalModuleType (x, _) -> inner (x : module_type :> any)
       | `CanonicalType (_, `Resolved _) -> false
       | `CanonicalType (x, _) -> inner (x : type_ :> any)
+      | `CanonicalDataType (_, `Resolved _) -> false
+      | `CanonicalDataType (x, _) -> inner (x : datatype :> any)
       | `OpaqueModule m -> inner (m :> any)
       | `OpaqueModuleType mt -> inner (mt :> any)
     in
@@ -677,51 +762,43 @@ module Path = struct
       | `Alias (dest, _src) -> parent_module_identifier dest
       | `OpaqueModule m -> parent_module_identifier m
 
+    and parent_datatype_identifier :
+        Paths_types.Resolved_path.datatype -> Identifier.DataType.t = function
+      | `Identifier id ->
+          (id : Identifier.Path.DataType.t :> Identifier.DataType.t)
+      | `CanonicalDataType (_, `Resolved p) -> parent_datatype_identifier p
+      | `CanonicalDataType (p, _) -> parent_datatype_identifier p
+      | `Type (m, n) -> Identifier.Mk.type_ (parent_module_identifier m, n)
+
     module Module = struct
       type t = Paths_types.Resolved_path.module_
 
       let is_hidden m =
         is_resolved_hidden (m : t :> Paths_types.Resolved_path.any)
-
-      let rec identifier : t -> Identifier.Path.Module.t =
-       fun x ->
-        let r = identifier in
-        match x with
-        | `Identifier id -> id
-        | `Subst (_, x) -> r x
-        | `Hidden p -> r p
-        | `Module (m, n) -> Identifier.Mk.module_ (parent_module_identifier m, n)
-        | `Canonical (p, _) -> r p
-        | `Apply (m, _) -> r m
-        | `Alias (dest, _src) -> r dest
-        | `OpaqueModule m -> r m
     end
 
     module ModuleType = struct
       type t = Paths_types.Resolved_path.module_type
-
-      let is_hidden m =
-        is_resolved_hidden (m : t :> Paths_types.Resolved_path.any)
     end
 
     module Type = struct
       type t = Paths_types.Resolved_path.type_
+    end
 
-      let of_ident id = `Identifier id
+    module DataType = struct
+      type t = Paths_types.Resolved_path.datatype
+    end
 
-      let is_hidden m =
-        is_resolved_hidden ~weak_canonical_test:false
-          (m : t :> Paths_types.Resolved_path.any)
+    module Constructor = struct
+      type t = Paths_types.Resolved_path.constructor
+    end
+
+    module Value = struct
+      type t = Paths_types.Resolved_path.value
     end
 
     module ClassType = struct
       type t = Paths_types.Resolved_path.class_type
-
-      let of_ident id = `Identifier id
-
-      let is_hidden m =
-        is_resolved_hidden ~weak_canonical_test:false
-          (m : t :> Paths_types.Resolved_path.any)
     end
 
     let rec identifier : t -> Identifier.t = function
@@ -733,6 +810,9 @@ module Path = struct
       | `Canonical (p, _) -> identifier (p :> t)
       | `Apply (m, _) -> identifier (m :> t)
       | `Type (m, n) -> Identifier.Mk.type_ (parent_module_identifier m, n)
+      | `Value (m, n) -> Identifier.Mk.value (parent_module_identifier m, n)
+      | `Constructor (m, n) ->
+          Identifier.Mk.constructor (parent_datatype_identifier m, n)
       | `ModuleType (m, n) ->
           Identifier.Mk.module_type (parent_module_identifier m, n)
       | `Class (m, n) -> Identifier.Mk.class_ (parent_module_identifier m, n)
@@ -752,8 +832,12 @@ module Path = struct
       | `CanonicalModuleType (p, _) -> identifier (p :> t)
       | `CanonicalType (_, `Resolved p) -> identifier (p :> t)
       | `CanonicalType (p, _) -> identifier (p :> t)
+      | `CanonicalDataType (_, `Resolved p) -> identifier (p :> t)
+      | `CanonicalDataType (p, _) -> identifier (p :> t)
       | `OpaqueModule m -> identifier (m :> t)
       | `OpaqueModuleType mt -> identifier (mt :> t)
+
+    let is_hidden r = is_resolved_hidden ~weak_canonical_test:false r
   end
 
   module Module = struct
@@ -768,6 +852,18 @@ module Path = struct
     type t = Paths_types.Path.type_
   end
 
+  module DataType = struct
+    type t = Paths_types.Path.datatype
+  end
+
+  module Constructor = struct
+    type t = Paths_types.Path.constructor
+  end
+
+  module Value = struct
+    type t = Paths_types.Path.value
+  end
+
   module ClassType = struct
     type t = Paths_types.Path.class_type
   end
@@ -781,92 +877,28 @@ module Fragment = struct
 
     type root = Paths_types.Resolved_fragment.root
 
-    let sig_of_mod m =
-      let open Paths_types.Resolved_fragment in
-      (m : module_ :> signature)
-
-    type base_name =
-      | Base of root
-      | Branch of ModuleName.t * Paths_types.Resolved_fragment.signature
-
-    let rec split_parent : Paths_types.Resolved_fragment.signature -> base_name
-        = function
-      | `Root i -> Base i
-      | `Subst (_, p) -> split_parent (sig_of_mod p)
-      | `Alias (_, p) -> split_parent (sig_of_mod p)
-      | `OpaqueModule m -> split_parent (sig_of_mod m)
-      | `Module (p, name) -> (
-          match split_parent p with
-          | Base i -> Branch (name, `Root i)
-          | Branch (base, m) -> Branch (base, `Module (m, name)))
-
     module Signature = struct
       type t = Paths_types.Resolved_fragment.signature
 
-      let rec split : t -> string * t option = function
-        | `Root _ -> ("", None)
-        | `Subst (_, p) -> split (sig_of_mod p)
-        | `Alias (_, p) -> split (sig_of_mod p)
-        | `OpaqueModule m -> split (sig_of_mod m)
-        | `Module (m, name) -> (
-            match split_parent m with
-            | Base _ -> (ModuleName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`Module (m, name))))
-
-      let rec identifier : t -> Identifier.Signature.t = function
+      let rec sgidentifier : t -> Identifier.Signature.t = function
         | `Root (`ModuleType i) -> Path.Resolved.parent_module_type_identifier i
         | `Root (`Module i) -> Path.Resolved.parent_module_identifier i
         | `Subst (s, _) -> Path.Resolved.parent_module_type_identifier s
         | `Alias (i, _) -> Path.Resolved.parent_module_identifier i
-        | `Module (m, n) -> Identifier.Mk.module_ (identifier m, n)
-        | `OpaqueModule m -> identifier (sig_of_mod m)
+        | `Module (m, n) -> Identifier.Mk.module_ (sgidentifier m, n)
+        | `OpaqueModule m -> sgidentifier (m :> t)
     end
 
     module Module = struct
       type t = Paths_types.Resolved_fragment.module_
-
-      let rec split : t -> string * t option = function
-        | `Subst (_, p) -> split p
-        | `Alias (_, p) -> split p
-        | `Module (m, name) -> (
-            match split_parent m with
-            | Base _ -> (ModuleName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`Module (m, name))))
-        | `OpaqueModule m -> split m
     end
 
     module ModuleType = struct
       type t = Paths_types.Resolved_fragment.module_type
-
-      let split : t -> string * t option = function
-        | `Module_type (m, name) -> (
-            match split_parent m with
-            | Base _ -> (ModuleTypeName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`Module_type (m, name))))
     end
 
     module Type = struct
       type t = Paths_types.Resolved_fragment.type_
-
-      let split : t -> string * t option = function
-        | `Type (m, name) -> (
-            match split_parent m with
-            | Base _ -> (TypeName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`Type (m, name))))
-        | `Class (m, name) -> (
-            match split_parent m with
-            | Base _ -> (ClassName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`Class (m, name))))
-        | `ClassType (m, name) -> (
-            match split_parent m with
-            | Base _ -> (ClassTypeName.to_string name, None)
-            | Branch (base, m) ->
-                (ModuleName.to_string base, Some (`ClassType (m, name))))
     end
 
     type leaf = Paths_types.Resolved_fragment.leaf
@@ -877,27 +909,20 @@ module Fragment = struct
       | `Subst (s, _) -> Path.Resolved.identifier (s :> Path.Resolved.t)
       | `Alias (p, _) ->
           (Path.Resolved.parent_module_identifier p :> Identifier.t)
-      | `Module (m, n) -> Identifier.Mk.module_ (Signature.identifier m, n)
+      | `Module (m, n) -> Identifier.Mk.module_ (Signature.sgidentifier m, n)
       | `Module_type (m, n) ->
-          Identifier.Mk.module_type (Signature.identifier m, n)
-      | `Type (m, n) -> Identifier.Mk.type_ (Signature.identifier m, n)
-      | `Class (m, n) -> Identifier.Mk.class_ (Signature.identifier m, n)
-      | `ClassType (m, n) -> Identifier.Mk.class_type (Signature.identifier m, n)
+          Identifier.Mk.module_type (Signature.sgidentifier m, n)
+      | `Type (m, n) -> Identifier.Mk.type_ (Signature.sgidentifier m, n)
+      | `Class (m, n) -> Identifier.Mk.class_ (Signature.sgidentifier m, n)
+      | `ClassType (m, n) ->
+          Identifier.Mk.class_type (Signature.sgidentifier m, n)
       | `OpaqueModule m -> identifier (m :> t)
 
     let rec is_hidden : t -> bool = function
-      | `Root (`ModuleType r) ->
-          Path.is_resolved_hidden ~weak_canonical_test:false
-            (r :> Path.Resolved.t)
-      | `Root (`Module r) ->
-          Path.is_resolved_hidden ~weak_canonical_test:false
-            (r :> Path.Resolved.t)
-      | `Subst (s, _) ->
-          Path.is_resolved_hidden ~weak_canonical_test:false
-            (s :> Path.Resolved.t)
-      | `Alias (s, _) ->
-          Path.is_resolved_hidden ~weak_canonical_test:false
-            (s :> Path.Resolved.t)
+      | `Root (`ModuleType r) -> Path.Resolved.(is_hidden (r :> t))
+      | `Root (`Module r) -> Path.Resolved.(is_hidden (r :> t))
+      | `Subst (s, _) -> Path.Resolved.(is_hidden (s :> t))
+      | `Alias (s, _) -> Path.Resolved.(is_hidden (s :> t))
       | `Module (m, _)
       | `Module_type (m, _)
       | `Type (m, _)
@@ -909,81 +934,20 @@ module Fragment = struct
 
   type t = Paths_types.Fragment.any
 
-  type base_name =
-    | Base of Resolved.root option
-    | Branch of ModuleName.t * Paths_types.Fragment.signature
-
-  let rec split_parent : Paths_types.Fragment.signature -> base_name = function
-    | `Root -> Base None
-    | `Resolved r -> (
-        match Resolved.split_parent r with
-        | Resolved.Base i -> Base (Some i)
-        | Resolved.Branch (base, m) -> Branch (base, `Resolved m))
-    | `Dot (m, name) -> (
-        match split_parent m with
-        | Base None -> Branch (ModuleName.make_std name, `Root)
-        | Base (Some i) -> Branch (ModuleName.make_std name, `Resolved (`Root i))
-        | Branch (base, m) -> Branch (base, `Dot (m, name)))
-
   module Signature = struct
     type t = Paths_types.Fragment.signature
-
-    let split : t -> string * t option = function
-      | `Root -> ("", None)
-      | `Resolved r ->
-          let base, m = Resolved.Signature.split r in
-          let m = match m with None -> None | Some m -> Some (`Resolved m) in
-          (base, m)
-      | `Dot (m, name) -> (
-          match split_parent m with
-          | Base _ -> (name, None)
-          | Branch (base, m) ->
-              (ModuleName.to_string base, Some (`Dot (m, name))))
   end
 
   module Module = struct
     type t = Paths_types.Fragment.module_
-
-    let split : t -> string * t option = function
-      | `Resolved r ->
-          let base, m = Resolved.Module.split r in
-          let m = match m with None -> None | Some m -> Some (`Resolved m) in
-          (base, m)
-      | `Dot (m, name) -> (
-          match split_parent m with
-          | Base _ -> (name, None)
-          | Branch (base, m) ->
-              (ModuleName.to_string base, Some (`Dot (m, name))))
   end
 
   module ModuleType = struct
     type t = Paths_types.Fragment.module_type
-
-    let split : t -> string * t option = function
-      | `Resolved r ->
-          let base, m = Resolved.ModuleType.split r in
-          let m = match m with None -> None | Some m -> Some (`Resolved m) in
-          (base, m)
-      | `Dot (m, name) -> (
-          match split_parent m with
-          | Base _ -> (name, None)
-          | Branch (base, m) ->
-              (ModuleName.to_string base, Some (`Dot (m, name))))
   end
 
   module Type = struct
     type t = Paths_types.Fragment.type_
-
-    let split : t -> string * t option = function
-      | `Resolved r ->
-          let base, m = Resolved.Type.split r in
-          let m = match m with None -> None | Some m -> Some (`Resolved m) in
-          (base, m)
-      | `Dot (m, name) -> (
-          match split_parent m with
-          | Base _ -> (name, None)
-          | Branch (base, m) ->
-              (ModuleName.to_string base, Some (`Dot (m, name))))
   end
 
   type leaf = Paths_types.Fragment.leaf
@@ -1000,14 +964,14 @@ module Reference = struct
       | `Identifier id -> id
       | `Hidden s -> parent_signature_identifier (s :> signature)
       | `Alias (sub, orig) ->
-          if Path.Resolved.Module.is_hidden ~weak_canonical_test:false sub then
+          if Path.Resolved.(is_hidden (sub :> t)) then
             parent_signature_identifier (orig :> signature)
           else
             (Path.Resolved.parent_module_identifier sub
               :> Identifier.Signature.t)
       | `AliasModuleType (sub, orig) ->
-          if Path.Resolved.ModuleType.is_hidden ~weak_canonical_test:false sub
-          then parent_signature_identifier (orig :> signature)
+          if Path.Resolved.(is_hidden (sub :> t)) then
+            parent_signature_identifier (orig :> signature)
           else
             (Path.Resolved.parent_module_type_identifier sub
               :> Identifier.Signature.t)
@@ -1028,32 +992,36 @@ module Reference = struct
       | `ClassType (sg, s) ->
           Identifier.Mk.class_type (parent_signature_identifier sg, s)
 
-    and parent_identifier : parent -> Identifier.Parent.t = function
+    and field_parent_identifier : field_parent -> Identifier.FieldParent.t =
+      function
       | `Identifier id -> id
       | (`Hidden _ | `Alias _ | `AliasModuleType _ | `Module _ | `ModuleType _)
         as sg ->
-          (parent_signature_identifier sg :> Identifier.Parent.t)
-      | `Type _ as t -> (parent_type_identifier t :> Identifier.Parent.t)
-      | (`Class _ | `ClassType _) as c ->
-          (parent_class_signature_identifier c :> Identifier.Parent.t)
+          (parent_signature_identifier sg :> Identifier.FieldParent.t)
+      | `Type _ as t -> (parent_type_identifier t :> Identifier.FieldParent.t)
 
     and label_parent_identifier : label_parent -> Identifier.LabelParent.t =
       function
       | `Identifier id -> id
+      | (`Class _ | `ClassType _) as c ->
+          (parent_class_signature_identifier c :> Identifier.LabelParent.t)
       | ( `Hidden _ | `Alias _ | `AliasModuleType _ | `Module _ | `ModuleType _
-        | `Type _ | `Class _ | `ClassType _ ) as r ->
-          (parent_identifier r :> Identifier.LabelParent.t)
+        | `Type _ ) as r ->
+          (field_parent_identifier r :> Identifier.LabelParent.t)
 
     and identifier : t -> Identifier.t = function
       | `Identifier id -> id
       | ( `Alias _ | `AliasModuleType _ | `Module _ | `Hidden _ | `Type _
         | `Class _ | `ClassType _ | `ModuleType _ ) as r ->
           (label_parent_identifier r :> Identifier.t)
-      | `Field (p, n) -> Identifier.Mk.field (parent_identifier p, n)
+      | `Field (p, n) -> Identifier.Mk.field (field_parent_identifier p, n)
       | `Constructor (s, n) ->
-          Identifier.Mk.constructor (parent_type_identifier s, n)
+          Identifier.Mk.constructor
+            ((parent_type_identifier s :> Identifier.DataType.t), n)
       | `Extension (p, q) ->
           Identifier.Mk.extension (parent_signature_identifier p, q)
+      | `ExtensionDecl (p, q, r) ->
+          Identifier.Mk.extension_decl (parent_signature_identifier p, (q, r))
       | `Exception (p, q) ->
           Identifier.Mk.exception_ (parent_signature_identifier p, q)
       | `Value (p, q) -> Identifier.Mk.value (parent_signature_identifier p, q)
@@ -1076,8 +1044,8 @@ module Reference = struct
       type t = Paths_types.Resolved_reference.datatype
     end
 
-    module Parent = struct
-      type t = Paths_types.Resolved_reference.parent
+    module FieldParent = struct
+      type t = Paths_types.Resolved_reference.field_parent
     end
 
     module LabelParent = struct
@@ -1108,6 +1076,10 @@ module Reference = struct
       type t = Paths_types.Resolved_reference.extension
     end
 
+    module ExtensionDecl = struct
+      type t = Paths_types.Resolved_reference.extension_decl
+    end
+
     module Exception = struct
       type t = Paths_types.Resolved_reference.exception_
     end
@@ -1157,8 +1129,8 @@ module Reference = struct
     type t = Paths_types.Reference.datatype
   end
 
-  module Parent = struct
-    type t = Paths_types.Reference.parent
+  module FragmentTypeParent = struct
+    type t = Paths_types.Reference.fragment_type_parent
   end
 
   module LabelParent = struct
@@ -1189,6 +1161,10 @@ module Reference = struct
     type t = Paths_types.Reference.extension
   end
 
+  module ExtensionDecl = struct
+    type t = Paths_types.Reference.extension_decl
+  end
+
   module Exception = struct
     type t = Paths_types.Reference.exception_
   end
