@@ -16,4 +16,18 @@
 
 open Odoc_document
 
-val renderer : Odoc_html.Config.t Renderer.t
+module Source : sig
+  type t = File of Fpath.t | Root of Fpath.t
+
+  val pp : Format.formatter -> t -> unit
+end
+
+type source = Source.t
+
+type args = {
+  html_config : Odoc_html.Config.t;
+  source : source option;
+  assets : Fpath.t list;
+}
+
+val renderer : args Renderer.t
