@@ -30,6 +30,12 @@ module PathTypeMap = Map.Make (struct
   let compare a b = Ident.compare (a :> Ident.any) (b :> Ident.any)
 end)
 
+module PathValueMap = Map.Make (struct
+  type t = Ident.path_value
+
+  let compare a b = Ident.compare (a :> Ident.any) (b :> Ident.any)
+end)
+
 module PathClassTypeMap = Map.Make (struct
   type t = Ident.path_class_type
 
@@ -76,6 +82,7 @@ module rec Module : sig
     | ModuleType of ModuleType.expr
 
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     type_ : decl;
     canonical : Odoc_model.Paths.Path.Module.t option;
@@ -147,6 +154,7 @@ and Extension : sig
   module Constructor : sig
     type t = {
       name : string;
+      locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
       doc : CComment.docs;
       args : TypeDecl.Constructor.argument;
       res : TypeExpr.t option;
@@ -165,6 +173,7 @@ end =
 
 and Exception : sig
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     args : TypeDecl.Constructor.argument;
     res : TypeExpr.t option;
@@ -228,6 +237,7 @@ and ModuleType : sig
     | TypeOf of typeof_t
 
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     canonical : Odoc_model.Paths.Path.ModuleType.t option;
     expr : expr option;
@@ -275,6 +285,7 @@ and TypeDecl : sig
   end
 
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     canonical : Odoc_model.Paths.Path.Type.t option;
     equation : Equation.t;
@@ -286,7 +297,12 @@ end =
 and Value : sig
   type value = Odoc_model.Lang.Value.value
 
-  type t = { doc : CComment.docs; type_ : TypeExpr.t; value : value }
+  type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
+    doc : CComment.docs;
+    type_ : TypeExpr.t;
+    value : value;
+  }
 end =
   Value
 
@@ -353,6 +369,7 @@ and Class : sig
     | Arrow of TypeExpr.label option * TypeExpr.t * decl
 
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     virtual_ : bool;
     params : TypeDecl.param list;
@@ -368,6 +385,7 @@ and ClassType : sig
     | Signature of ClassSignature.t
 
   type t = {
+    locs : Odoc_model.Paths.Identifier.SourceLocation.t option;
     doc : CComment.docs;
     virtual_ : bool;
     params : TypeDecl.param list;
@@ -465,7 +483,7 @@ and Label : sig
   type t = {
     attrs : Odoc_model.Comment.heading_attrs;
     label : Ident.label;
-    text : Odoc_model.Comment.link_content;
+    text : Odoc_model.Comment.paragraph;
     location : Odoc_model.Location_.span;
   }
 end =
@@ -478,7 +496,7 @@ module Element = struct
 
   type module_type = [ `ModuleType of Identifier.ModuleType.t * ModuleType.t ]
 
-  type type_ = [ `Type of Identifier.Type.t * TypeDecl.t ]
+  type datatype = [ `Type of Identifier.Type.t * TypeDecl.t ]
 
   type value = [ `Value of Identifier.Value.t * Value.t ]
 
@@ -488,7 +506,7 @@ module Element = struct
 
   type class_type = [ `ClassType of Identifier.ClassType.t * ClassType.t ]
 
-  type datatype = [ type_ | class_ | class_type ]
+  type type_ = [ datatype | class_ | class_type ]
 
   type signature = [ module_ | module_type ]
 
@@ -498,25 +516,32 @@ module Element = struct
   type exception_ = [ `Exception of Identifier.Exception.t * Exception.t ]
 
   type extension =
-    [ `Extension of Identifier.Extension.t * Extension.Constructor.t ]
+    [ `Extension of
+      Identifier.Extension.t * Extension.Constructor.t * Extension.t ]
+
+  type extension_decl =
+    [ `ExtensionDecl of Identifier.Extension.t * Extension.Constructor.t ]
 
   type field = [ `Field of Identifier.Field.t * TypeDecl.Field.t ]
 
   (* No component for pages yet *)
   type page = [ `Page of Identifier.Page.t * Odoc_model.Lang.Page.t ]
 
-  type label_parent = [ signature | datatype | page ]
+  type label_parent = [ signature | type_ | page ]
+
+  type fragment_type_parent = [ signature | datatype ]
 
   type any =
     [ signature
     | value
-    | type_
+    | datatype
     | label
     | class_
     | class_type
     | constructor
     | exception_
     | extension
+    | extension_decl
     | field
     | page ]
 
@@ -533,7 +558,8 @@ module Element = struct
     | `Constructor (id, _) -> (id :> t)
     | `Exception (id, _) -> (id :> t)
     | `Field (id, _) -> (id :> t)
-    | `Extension (id, _) -> (id :> t)
+    | `Extension (id, _, _) -> (id :> t)
+    | `ExtensionDecl (id, _) -> (id :> t)
     | `Page (id, _) -> (id :> t)
 end
 
@@ -1010,6 +1036,39 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" resolved_parent_path p
           (Odoc_model.Names.TypeName.to_string t)
 
+  and resolved_datatype_path :
+      Format.formatter -> Cpath.Resolved.datatype -> unit =
+   fun ppf p ->
+    match p with
+    | `Local id -> Format.fprintf ppf "%a" Ident.fmt id
+    | `Gpath p ->
+        Format.fprintf ppf "%a" model_resolved_path
+          (p :> Odoc_model.Paths.Path.Resolved.t)
+    | `Substituted x ->
+        Format.fprintf ppf "substituted(%a)" resolved_datatype_path x
+    | `CanonicalDataType (t1, t2) ->
+        Format.fprintf ppf "canonicalty(%a,%a)" resolved_datatype_path t1
+          model_path
+          (t2 :> Odoc_model.Paths.Path.t)
+    | `Type (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_parent_path p
+          (Odoc_model.Names.TypeName.to_string t)
+
+  and resolved_value_path : Format.formatter -> Cpath.Resolved.value -> unit =
+   fun ppf p ->
+    match p with
+    | `Value (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_parent_path p
+          (Odoc_model.Names.ValueName.to_string t)
+
+  and resolved_constructor_path :
+      Format.formatter -> Cpath.Resolved.constructor -> unit =
+   fun ppf p ->
+    match p with
+    | `Constructor (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_datatype_path p
+          (Odoc_model.Names.ConstructorName.to_string t)
+
   and resolved_parent_path : Format.formatter -> Cpath.Resolved.parent -> unit =
    fun ppf p ->
     match p with
@@ -1038,6 +1097,39 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" resolved_parent_path p
           (Odoc_model.Names.TypeName.to_string t)
 
+  and datatype_path : Format.formatter -> Cpath.datatype -> unit =
+   fun ppf p ->
+    match p with
+    | `Resolved r -> Format.fprintf ppf "r(%a)" resolved_datatype_path r
+    | `Identifier (id, b) ->
+        Format.fprintf ppf "identifier(%a, %b)" model_identifier
+          (id :> Odoc_model.Paths.Identifier.t)
+          b
+    | `Local (id, b) -> Format.fprintf ppf "local(%a,%b)" Ident.fmt id b
+    | `Substituted s -> Format.fprintf ppf "substituted(%a)" datatype_path s
+    | `Dot (m, s) -> Format.fprintf ppf "%a.%s" module_path m s
+    | `Type (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_parent_path p
+          (Odoc_model.Names.TypeName.to_string t)
+
+  and value_path : Format.formatter -> Cpath.value -> unit =
+   fun ppf p ->
+    match p with
+    | `Resolved r -> Format.fprintf ppf "r(%a)" resolved_value_path r
+    | `Dot (m, s) -> Format.fprintf ppf "%a.%s" module_path m s
+    | `Value (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_parent_path p
+          (Odoc_model.Names.ValueName.to_string t)
+
+  and constructor_path : Format.formatter -> Cpath.constructor -> unit =
+   fun ppf p ->
+    match p with
+    | `Resolved r -> Format.fprintf ppf "r(%a)" resolved_constructor_path r
+    | `Dot (m, s) -> Format.fprintf ppf "%a.%s" datatype_path m s
+    | `Constructor (p, t) ->
+        Format.fprintf ppf "%a.%s" resolved_datatype_path p
+          (Odoc_model.Names.ConstructorName.to_string t)
+
   and resolved_class_type_path :
       Format.formatter -> Cpath.Resolved.class_type -> unit =
    fun ppf p ->
@@ -1111,6 +1203,14 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" model_resolved_path
           (parent :> t)
           (Odoc_model.Names.TypeName.to_string name)
+    | `Constructor (parent, name) ->
+        Format.fprintf ppf "%a.%s" model_resolved_path
+          (parent :> t)
+          (Odoc_model.Names.ConstructorName.to_string name)
+    | `Value (parent, name) ->
+        Format.fprintf ppf "%a.%s" model_resolved_path
+          (parent :> t)
+          (Odoc_model.Names.ValueName.to_string name)
     | `Alias (dest, src) ->
         Format.fprintf ppf "alias(%a,%a)" model_resolved_path
           (dest :> t)
@@ -1141,6 +1241,11 @@ module Fmt = struct
           (t1 :> t)
           model_path
           (t2 :> Odoc_model.Paths.Path.t)
+    | `CanonicalDataType (t1, t2) ->
+        Format.fprintf ppf "canonicaldaty(%a,%a)" model_resolved_path
+          (t1 :> t)
+          model_path
+          (t2 :> Odoc_model.Paths.Path.t)
     | `Apply (funct, arg) ->
         Format.fprintf ppf "%a(%a)" model_resolved_path
           (funct :> t)
@@ -1233,8 +1338,31 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" model_identifier
           (p :> Odoc_model.Paths.Identifier.t)
           (ExtensionName.to_string name)
+    | `ExtensionDecl (p, _, name) ->
+        Format.fprintf ppf "%a.%s" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+          (ExtensionName.to_string name)
     | `Page (_, name) | `LeafPage (_, name) ->
         Format.fprintf ppf "%s" (PageName.to_string name)
+    | `SourcePage (p, name) | `SourceDir (p, name) ->
+        Format.fprintf ppf "%a/%s" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+          name
+    | `SourceLocation (p, def) ->
+        Format.fprintf ppf "%a#%s" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+          (DefName.to_string def)
+    | `SourceLocationInternal (p, def) ->
+        Format.fprintf ppf "%a#%s" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+          (LocalName.to_string def)
+    | `SourceLocationMod p ->
+        Format.fprintf ppf "%a#" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+    | `AssetFile (p, name) ->
+        Format.fprintf ppf "%a/%s" model_identifier
+          (p :> Odoc_model.Paths.Identifier.t)
+          name
 
   and model_fragment ppf (f : Odoc_model.Paths.Fragment.t) =
     match f with
@@ -1380,6 +1508,10 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" model_resolved_reference
           (parent :> t)
           (ExtensionName.to_string name)
+    | `ExtensionDecl (parent, name, _) ->
+        Format.fprintf ppf "%a.%s" model_resolved_reference
+          (parent :> t)
+          (ExtensionName.to_string name)
     | `Exception (parent, name) ->
         Format.fprintf ppf "%a.%s" model_resolved_reference
           (parent :> t)
@@ -1450,6 +1582,10 @@ module Fmt = struct
         Format.fprintf ppf "%a.%s" model_reference
           (parent :> t)
           (ExtensionName.to_string name)
+    | `ExtensionDecl (parent, name) ->
+        Format.fprintf ppf "%a.%s" model_reference
+          (parent :> t)
+          (ExtensionName.to_string name)
     | `Exception (parent, name) ->
         Format.fprintf ppf "%a.%s" model_reference
           (parent :> t)
@@ -1721,6 +1857,31 @@ module Of_Lang = struct
     | `ClassType (p, name) ->
         `ClassType (`Module (resolved_module_path ident_map p), name)
 
+  and resolved_datatype_path :
+      _ -> Odoc_model.Paths.Path.Resolved.DataType.t -> Cpath.Resolved.datatype
+      =
+   fun ident_map p ->
+    match p with
+    | `Identifier i -> (
+        match identifier Maps.Type.find ident_map.types i with
+        | `Local l -> `Local l
+        | `Identifier _ -> `Gpath p)
+    | `CanonicalDataType (p1, p2) ->
+        `CanonicalDataType (resolved_datatype_path ident_map p1, p2)
+    | `Type (p, name) -> `Type (`Module (resolved_module_path ident_map p), name)
+
+  and resolved_value_path :
+      _ -> Odoc_model.Paths.Path.Resolved.Value.t -> Cpath.Resolved.value =
+   fun ident_map (`Value (p, name)) ->
+    `Value (`Module (resolved_module_path ident_map p), name)
+
+  and resolved_constructor_path :
+      _ ->
+      Odoc_model.Paths.Path.Resolved.Constructor.t ->
+      Cpath.Resolved.constructor =
+   fun ident_map (`Constructor (p, name)) ->
+    `Constructor (resolved_datatype_path ident_map p, name)
+
   and resolved_class_type_path :
       _ ->
       Odoc_model.Paths.Path.Resolved.ClassType.t ->
@@ -1773,6 +1934,29 @@ module Of_Lang = struct
         | `Local i -> `Local (i, b))
     | `Dot (path', x) -> `Dot (module_path ident_map path', x)
 
+  and value_path : _ -> Odoc_model.Paths.Path.Value.t -> Cpath.value =
+   fun ident_map p ->
+    match p with
+    | `Resolved r -> `Resolved (resolved_value_path ident_map r)
+    | `Dot (path', x) -> `Dot (module_path ident_map path', x)
+
+  and datatype : _ -> Odoc_model.Paths.Path.DataType.t -> Cpath.datatype =
+   fun ident_map p ->
+    match p with
+    | `Resolved r -> `Resolved (resolved_datatype_path ident_map r)
+    | `Identifier (i, b) -> (
+        match identifier Maps.Type.find ident_map.types i with
+        | `Identifier i -> `Identifier (i, b)
+        | `Local i -> `Local (i, b))
+    | `Dot (path', x) -> `Dot (module_path ident_map path', x)
+
+  and constructor_path :
+      _ -> Odoc_model.Paths.Path.Constructor.t -> Cpath.constructor =
+   fun ident_map p ->
+    match p with
+    | `Resolved r -> `Resolved (resolved_constructor_path ident_map r)
+    | `Dot (path', x) -> `Dot (datatype ident_map path', x)
+
   and class_type_path :
       _ -> Odoc_model.Paths.Path.ClassType.t -> Cpath.class_type =
    fun ident_map p ->
@@ -1864,7 +2048,8 @@ module Of_Lang = struct
   let rec type_decl ident_map ty =
     let open Odoc_model.Lang.TypeDecl in
     {
-      TypeDecl.doc = docs ident_map ty.doc;
+      TypeDecl.locs = ty.locs;
+      doc = docs ident_map ty.doc;
       canonical = ty.canonical;
       equation = type_equation ident_map ty.equation;
       representation =
@@ -1989,10 +2174,10 @@ module Of_Lang = struct
 
   and module_decl ident_map m =
     match m with
-    | Odoc_model.Lang.Module.Alias (p, e) ->
+    | Lang.Module.Alias (p, e) ->
         Module.Alias
           (module_path ident_map p, option simple_expansion ident_map e)
-    | Odoc_model.Lang.Module.ModuleType s ->
+    | Lang.Module.ModuleType s ->
         Module.ModuleType (module_type_expr ident_map s)
 
   and include_decl ident_map m =
@@ -2028,7 +2213,13 @@ module Of_Lang = struct
   and module_ ident_map m =
     let type_ = module_decl ident_map m.Odoc_model.Lang.Module.type_ in
     let canonical = m.Odoc_model.Lang.Module.canonical in
-    { Module.doc = docs ident_map m.doc; type_; canonical; hidden = m.hidden }
+    {
+      Module.locs = m.locs;
+      doc = docs ident_map m.doc;
+      type_;
+      canonical;
+      hidden = m.hidden;
+    }
 
   and with_module_type_substitution ident_map m =
     let open Odoc_model.Lang.ModuleType in
@@ -2078,6 +2269,7 @@ module Of_Lang = struct
     let res = Opt.map (type_expression ident_map) c.res in
     {
       Extension.Constructor.name = Paths.Identifier.name c.id;
+      locs = c.locs;
       doc = docs ident_map c.doc;
       args;
       res;
@@ -2087,7 +2279,7 @@ module Of_Lang = struct
     let open Odoc_model.Lang.Exception in
     let args = type_decl_constructor_argument ident_map e.args in
     let res = Opt.map (type_expression ident_map) e.res in
-    { Exception.doc = docs ident_map e.doc; args; res }
+    { Exception.locs = e.locs; doc = docs ident_map e.doc; args; res }
 
   and u_module_type_expr ident_map m =
     let open Odoc_model in
@@ -2169,11 +2361,16 @@ module Of_Lang = struct
     let expr =
       Opt.map (module_type_expr ident_map) m.Odoc_model.Lang.ModuleType.expr
     in
-    { ModuleType.doc = docs ident_map m.doc; canonical = m.canonical; expr }
+    {
+      ModuleType.locs = m.locs;
+      doc = docs ident_map m.doc;
+      canonical = m.canonical;
+      expr;
+    }
 
   and value ident_map v =
     let type_ = type_expression ident_map v.Lang.Value.type_ in
-    { Value.type_; doc = docs ident_map v.doc; value = v.value }
+    { Value.type_; doc = docs ident_map v.doc; value = v.value; locs = v.locs }
 
   and include_ ident_map i =
     let open Odoc_model.Lang.Include in
@@ -2193,7 +2390,8 @@ module Of_Lang = struct
     let open Odoc_model.Lang.Class in
     let expansion = Opt.map (class_signature ident_map) c.expansion in
     {
-      Class.doc = docs ident_map c.doc;
+      Class.locs = c.locs;
+      doc = docs ident_map c.doc;
       virtual_ = c.virtual_;
       params = c.params;
       type_ = class_decl ident_map c.type_;
@@ -2219,7 +2417,8 @@ module Of_Lang = struct
     let open Odoc_model.Lang.ClassType in
     let expansion = Opt.map (class_signature ident_map) t.expansion in
     {
-      ClassType.doc = docs ident_map t.doc;
+      ClassType.locs = t.locs;
+      doc = docs ident_map t.doc;
       virtual_ = t.virtual_;
       params = t.params;
       expr = class_type_expr ident_map t.expr;
@@ -2296,11 +2495,11 @@ module Of_Lang = struct
   and module_of_module_substitution ident_map
       (t : Odoc_model.Lang.ModuleSubstitution.t) =
     let manifest = module_path ident_map t.manifest in
-    let canonical = None in
     {
-      Module.doc = docs ident_map t.doc;
+      Module.locs = None;
+      doc = docs ident_map t.doc;
       type_ = Alias (manifest, None);
-      canonical;
+      canonical = None;
       hidden = false;
     }
 
@@ -2327,59 +2526,59 @@ module Of_Lang = struct
     let items =
       List.rev_map
         (let open Odoc_model.Lang.Signature in
-        let open Odoc_model.Paths in
-        function
-        | Type (r, t) ->
-            let id = Identifier.Maps.Type.find t.id ident_map.types in
-            let t' = Delayed.put (fun () -> type_decl ident_map t) in
-            Signature.Type (id, r, t')
-        | TypeSubstitution t ->
-            let id = Identifier.Maps.Type.find t.id ident_map.types in
-            let t' = type_decl ident_map t in
-            Signature.TypeSubstitution (id, t')
-        | Module (r, m) ->
-            let id =
-              Identifier.Maps.Module.find
-                (m.id :> Identifier.Module.t)
-                ident_map.modules
-            in
-            let m' = Delayed.put (fun () -> module_ ident_map m) in
-            Signature.Module (id, r, m')
-        | ModuleSubstitution m ->
-            let id = Identifier.Maps.Module.find m.id ident_map.modules in
-            let m' = module_substitution ident_map m in
-            Signature.ModuleSubstitution (id, m')
-        | ModuleTypeSubstitution m ->
-            let id =
-              Identifier.Maps.ModuleType.find m.id ident_map.module_types
-            in
-            let m' = module_type_substitution ident_map m in
-            Signature.ModuleTypeSubstitution (id, m')
-        | ModuleType m ->
-            let id =
-              Identifier.Maps.ModuleType.find m.id ident_map.module_types
-            in
-            let m' = Delayed.put (fun () -> module_type ident_map m) in
-            Signature.ModuleType (id, m')
-        | Value v ->
-            let id = Ident.Of_Identifier.value v.id in
-            let v' = Delayed.put (fun () -> value ident_map v) in
-            Signature.Value (id, v')
-        | Comment c -> Comment (docs_or_stop ident_map c)
-        | TypExt e -> TypExt (extension ident_map e)
-        | Exception e ->
-            let id = Ident.Of_Identifier.exception_ e.id in
-            Exception (id, exception_ ident_map e)
-        | Class (r, c) ->
-            let id = Identifier.Maps.Class.find c.id ident_map.classes in
-            Class (id, r, class_ ident_map c)
-        | ClassType (r, c) ->
-            let id =
-              Identifier.Maps.ClassType.find c.id ident_map.class_types
-            in
-            ClassType (id, r, class_type ident_map c)
-        | Open o -> Open (open_ ident_map o)
-        | Include i -> Include (include_ ident_map i))
+         let open Odoc_model.Paths in
+         function
+         | Type (r, t) ->
+             let id = Identifier.Maps.Type.find t.id ident_map.types in
+             let t' = Delayed.put (fun () -> type_decl ident_map t) in
+             Signature.Type (id, r, t')
+         | TypeSubstitution t ->
+             let id = Identifier.Maps.Type.find t.id ident_map.types in
+             let t' = type_decl ident_map t in
+             Signature.TypeSubstitution (id, t')
+         | Module (r, m) ->
+             let id =
+               Identifier.Maps.Module.find
+                 (m.id :> Identifier.Module.t)
+                 ident_map.modules
+             in
+             let m' = Delayed.put (fun () -> module_ ident_map m) in
+             Signature.Module (id, r, m')
+         | ModuleSubstitution m ->
+             let id = Identifier.Maps.Module.find m.id ident_map.modules in
+             let m' = module_substitution ident_map m in
+             Signature.ModuleSubstitution (id, m')
+         | ModuleTypeSubstitution m ->
+             let id =
+               Identifier.Maps.ModuleType.find m.id ident_map.module_types
+             in
+             let m' = module_type_substitution ident_map m in
+             Signature.ModuleTypeSubstitution (id, m')
+         | ModuleType m ->
+             let id =
+               Identifier.Maps.ModuleType.find m.id ident_map.module_types
+             in
+             let m' = Delayed.put (fun () -> module_type ident_map m) in
+             Signature.ModuleType (id, m')
+         | Value v ->
+             let id = Ident.Of_Identifier.value v.id in
+             let v' = Delayed.put (fun () -> value ident_map v) in
+             Signature.Value (id, v')
+         | Comment c -> Comment (docs_or_stop ident_map c)
+         | TypExt e -> TypExt (extension ident_map e)
+         | Exception e ->
+             let id = Ident.Of_Identifier.exception_ e.id in
+             Exception (id, exception_ ident_map e)
+         | Class (r, c) ->
+             let id = Identifier.Maps.Class.find c.id ident_map.classes in
+             Class (id, r, class_ ident_map c)
+         | ClassType (r, c) ->
+             let id =
+               Identifier.Maps.ClassType.find c.id ident_map.class_types
+             in
+             ClassType (id, r, class_type ident_map c)
+         | Open o -> Open (open_ ident_map o)
+         | Include i -> Include (include_ ident_map i))
         sg.items
       |> List.rev
     in
@@ -2405,7 +2604,8 @@ end
 
 let module_of_functor_argument (arg : FunctorParameter.parameter) =
   {
-    Module.doc = [];
+    Module.locs = None;
+    doc = [];
     type_ = ModuleType arg.expr;
     canonical = None;
     hidden = false;
